/*$6 ***************************************************************************
Copyright (C) <2007> <Sven Trenkel, Rene Weller, Gabriel Zachmann>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.
 ******************************************************************************* */
/**
 *  \brief Defines interfaces for library wrappers
 *
 *  \author Sven Trenkel, Rene Weller, Gabriel Zachmann
 */
 
#ifndef BENCH_H
#define BENCH_H

#include <OpenSG/OSGGeometry.h>
#include <OpenSG/OSGVector.h>

/**
 *  \Each Library must provide the following functions
 *   - prepareObjects
 *   - doCollisions
 *  getDistance and shutdown are optional
 */
typedef struct
{
    void ( *prepareObjects ) ( int num, osg::NodePtr *object ); //transfer the OSG-Nodes into special format of the library
    int ( *doCollisions ) ( int num, osg::Matrix *m ); //transform object num with Matrix m and do a collision check
    double ( *getDistance ) 
        (
            int num,
            osg::Matrix *m,
            osg::Pnt3f *pos1,
            osg::Pnt3f *pos2
        ); //return distance between moving and fixed objects together with the closest points pos1 and pos2 (optional)
    void ( *shutdown ) (); //shutdown library (optional)
}
tAPI;


/**
 *  \brief Algorithm struct
 */
typedef struct  Algorithm
{
    const char          *shortName; //short name of the algorithm will be written in the result-files
    const char          *name; //full name of the algorithm. will be displayed with the command-line options
    tAPI                api; //API-struct, see above for details
    struct Algorithm    *next; //pointer to next algorithm-struct
} Algorithm;
#endif

