/*$6 ***************************************************************************
Copyright (C) <2007> <Sven Trenkel, Rene Weller, Gabriel Zachmann>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.
 ******************************************************************************* */
/**
 *  \brief Collection of Utility functions
 *
 *  \author Sven Trenkel, Rene Weller, Gabriel Zachmann
 */

#ifndef UTILS_H
#define UTILS_H
#if defined( __sgi ) || defined( _WIN32 )
#pragma once
#endif

/*
 * Includes ;
 */
#include <math.h>

#include <OpenSG/OSGConfig.h>
#include <OpenSG/OSGBaseTypes.h>
#include <OpenSG/OSGBaseFunctions.h>
#include <OpenSG/OSGVector.h>
#include <OpenSG/OSGMatrix.h>
#include <OpenSG/OSGGeoProperty.h>
#include <OpenSG/OSGQuaternion.h>
#include <OpenSG/OSGGeometry.h>

#include "Settings.h"

using osg::         Vec3f;
using osg::         Vec4f;
using osg::         Pnt3f;
using osg::         MFPnt3f;
using osg::         NodePtr;
using osg::         GeometryPtr;

using namespace     std;

/*
 * Constants
 */
 
 
/* Epsilon: all collision detection math will use this threshold */
const float         NearZero = 1E-7;

/*
 * Forward declarations ;
 * ;
 * Functions: See definitons for details
 */
osg::GeometryPtr    getGeom( const osg::NodePtr node );
void                merge( const osg::NodePtr &original, osg::NodePtr &copy );
float               getTime( void );
osg::MFPnt3f*       getPoints ( const osg::GeometryPtr geo );
void            getTransfomUpto( const osg::NodePtr  &cur,
                                 const osg::NodePtr  &upto,
                                 osg::Matrix         &result );
osg::NodePtr    findGeomNode( const osg::NodePtr node );
void            iterFaces( const osg::NodePtr  &node,
                           void (*callback)
                               (
                               const osg::NodePtr &,
                           const osg::GeometryPtr &,
                           const osg::FaceIterator &,
                           void *
                       ),
                           void *data );
void            getAABB( const osg::GeometryPtr &geom, Pnt3f &Min, Pnt3f &Max );
osg::Pnt3f              lincomb( float               c1,
                                 const osg::Pnt3f    &pnt1,
                                 float               c2,
                                 const osg::Pnt3f    &pnt2 );
osg::Matrix             axisToMat( const osg::Vec3f &a, float d );
osg::Matrix operator    *( const osg::Matrix &m1, const osg::Matrix &m2 );

osg::NodePtr            loadGeom( const char *filename );
osg::NodePtr            createGeom( GeometryTypeE   type,
                                    unsigned int    complexity );
unsigned countTriangles( osg::GeometryPtr geo, double *avArea );

char                    *getToken( char **buf, const char *trenner );
osg::NodePtr            createDistanceObject( unsigned int mode, osg::NodePtr node, double dist );

/* timing functions */
float           usertime( void );

class           NanoTimer
{
public:
    NanoTimer();

    void            start( void );
    double          elapsed( void ) const;

    static bool     usesHighFrequ( void );
    static double   frequ( void );

private:
    unsigned long long int          m_time_stamp;

    static bool                     M_Use_High_Frequ;
    static double                   M_GHz;
    static bool                     M_FrequencyChecked;

    static long long unsigned int   getTimeStamp( void );
    static void                     checkFrequency( void );
};

/* -------------------------------------------------------------------------
*  Class for triangle representation
--------------------------------------------------------------------------- */

class   Tri
{

public:
    double  triArea();

    Tri()
    {
    }
    Tri( osg::GeoPositionsPtr ptr, int i1, int i2, int i3 ) :
    coordPtr( ptr )
    {
        idx[0] = i1;
        idx[1] = i2;
        idx[2] = i3;
        triArea();
    }

    osg::GeoPositionsPtr    coordPtr;
    int                     idx[3];
    int                     pos;
    double                  area;
};

#endif /* UTILS_H */
